package techreborn.client;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.network.IGuiHandler;
import reborncore.api.tile.IContainerLayout;
import reborncore.common.container.RebornContainer;
import techreborn.client.container.*;
import techreborn.client.gui.*;
import techreborn.manual.GuiManual;
import techreborn.tiles.*;
import techreborn.tiles.fusionReactor.TileEntityFusionController;
import techreborn.tiles.generator.*;
import techreborn.tiles.idsu.TileIDSU;
import techreborn.tiles.lesu.TileLesu;
import techreborn.tiles.multiblock.*;
import techreborn.tiles.storage.TileBatBox;
import techreborn.tiles.storage.TileMFE;
import techreborn.tiles.storage.TileMFSU;
import techreborn.tiles.teir1.*;

public class GuiHandler implements IGuiHandler
{

	public static final int thermalGeneratorID = 0;
	public static final int quantumTankID = 1;
	public static final int quantumChestID = 2;
	public static final int centrifugeID = 3;
	public static final int rollingMachineID = 4;
	public static final int blastFurnaceID = 5;
	public static final int alloySmelterID = 6;
	public static final int industrialGrinderID = 7;
	public static final int implosionCompresserID = 8;
	public static final int matterfabID = 9;
	public static final int manuelID = 10;
	public static final int chunkloaderID = 11;
	public static final int assemblingmachineID = 12;
	public static final int dieselGeneratorID = 15;
	public static final int industrialElectrolyzerID = 16;
	public static final int aesuID = 17;
	public static final int alloyFurnaceID = 18;
	public static final int sawMillID = 19;
	public static final int chemicalReactorID = 20;
	public static final int semifluidGeneratorID = 21;
	public static final int gasTurbineID = 22;
	public static final int digitalChestID = 23;
	public static final int destructoPackID = 25;
	public static final int lesuID = 26;
	public static final int idsuID = 27;
	public static final int chargeBench = 28;
	public static final int fusionID = 29;
	public static final int vacuumFreezerID = 30;
	public static final int grinderID = 31;
	public static final int generatorID = 32;
	public static final int extractorID = 33;
	public static final int compressorID = 34;
	public static final int electricFurnaceID = 35;
	public static final int ironFurnace = 36;
	public static final int recyclerID = 37;
	public static final int scrapboxinatorID = 38;
	public static final int batboxID = 39;
	public static final int mfsuID = 40;
	public static final int mfeID = 41;
	@Override
	public Object getServerGuiElement(int ID, EntityPlayer player, World world, int x, int y, int z)
	{
		RebornContainer container = null;
		if (ID == thermalGeneratorID)
		{
			return new ContainerThermalGenerator((TileThermalGenerator) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == semifluidGeneratorID)
		{
			return new ContainerSemifluidGenerator((TileSemifluidGenerator) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == gasTurbineID)
		{
			return new ContainerGasTurbine((TileGasTurbine) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == quantumTankID)
		{
			return new ContainerQuantumTank((TileQuantumTank) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == digitalChestID)
		{
			return new ContainerDigitalChest((TileDigitalChest) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == quantumChestID)
		{
			return new ContainerQuantumChest((TileQuantumChest) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == centrifugeID)
		{
			container = new ContainerCentrifuge();
		} else if (ID == rollingMachineID)
		{
			return new ContainerRollingMachine((TileRollingMachine) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == blastFurnaceID)
		{
			return new ContainerBlastFurnace((TileBlastFurnace) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == alloySmelterID)
		{
			container = new ContainerAlloySmelter();
		} else if (ID == industrialGrinderID)
		{
			return new ContainerIndustrialGrinder((TileIndustrialGrinder) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == implosionCompresserID)
		{
			return new ContainerImplosionCompressor(
					(TileImplosionCompressor) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == matterfabID)
		{
			return new ContainerMatterFabricator((TileMatterFabricator) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == chunkloaderID)
		{
			return new ContainerChunkloader((TileChunkLoader) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == assemblingmachineID)
		{
			container = new ContainerAssemblingMachine();
		} else if (ID == dieselGeneratorID)
		{
			return new ContainerDieselGenerator((TileDieselGenerator) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == industrialElectrolyzerID)
		{
			return new ContainerIndustrialElectrolyzer(
					(TileIndustrialElectrolyzer) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == aesuID)
		{
			return new ContainerAESU((TileAesu) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == alloyFurnaceID)
		{
			return new ContainerAlloyFurnace((TileAlloyFurnace) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == sawMillID)
		{
			return new ContainerIndustrialSawmill((TileIndustrialSawmill) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == chemicalReactorID)
		{
			return new ContainerChemicalReactor((TileChemicalReactor) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == manuelID)
		{
			return null;
		} else if (ID == destructoPackID)
		{
			return new ContainerDestructoPack(player);
		} else if (ID == lesuID)
		{
			return new ContainerLESU((TileLesu) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == idsuID)
		{
			return new ContainerIDSU((TileIDSU) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == chargeBench)
		{
			return new ContainerChargeBench((TileChargeBench) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == fusionID)
		{
			return new ContainerFusionReactor((TileEntityFusionController) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == vacuumFreezerID)
		{
			return new ContainerVacuumFreezer((TileVacuumFreezer) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == grinderID)
		{
			container = new ContainerGrinder();
		} else if (ID == generatorID)
		{
			return new ContainerGenerator((TileGenerator) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == extractorID)
		{
			container = new ContainerExtractor();
		} else if (ID == compressorID)
		{
			return new ContainerCompressor((TileCompressor) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == electricFurnaceID)
		{
			return new ContainerElectricFurnace((TileElectricFurnace) world.func_175625_s(new BlockPos(x, y, z)),
					player);
		} else if (ID == ironFurnace)
		{
			return new ContainerIronFurnace((TileIronFurnace) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == recyclerID)
		{
			return new ContainerRecycler((TileRecycler) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == scrapboxinatorID)
		{
			return new ContainerScrapboxinator((TileScrapboxinator) world.func_175625_s(new BlockPos(x, y, z)), player);
		} else if (ID == batboxID)
		{
			return new ContainerBatbox((TileBatBox) world.func_175625_s(new BlockPos(x, y, z)), player);
		}else if (ID == mfsuID){
			return new ContainerMFSU((TileMFSU) world.func_175625_s(new BlockPos(x, y, z)), player);
		}else if (ID == mfeID){
			return new ContainerMFE((TileMFE) world.func_175625_s(new BlockPos(x, y, z)), player);
		}
		if(container != null){
			if(container instanceof IContainerLayout){
				IContainerLayout layout = (IContainerLayout) container;
				layout.setTile(world.func_175625_s(new BlockPos(x, y, z)));
				layout.setPlayer(player);
				layout.addInventorySlots();
				layout.addPlayerSlots();
			}
			return container;
		}
		return null;
	}

	@Override
	public Object getClientGuiElement(int ID, EntityPlayer player, World world, int x, int y, int z)
	{
		if (ID == thermalGeneratorID)
		{
			return new GuiThermalGenerator(player, (TileThermalGenerator) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == semifluidGeneratorID)
		{
			return new GuiSemifluidGenerator(player,
					(TileSemifluidGenerator) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == gasTurbineID)
		{
			return new GuiGasTurbine(player, (TileGasTurbine) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == quantumTankID)
		{
			return new GuiQuantumTank(player, (TileQuantumTank) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == digitalChestID)
		{
			return new GuiDigitalChest(player, (TileDigitalChest) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == quantumChestID)
		{
			return new GuiQuantumChest(player, (TileQuantumChest) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == centrifugeID)
		{
			return new GuiCentrifuge(player, (TileCentrifuge) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == rollingMachineID)
		{
			return new GuiRollingMachine(player, (TileRollingMachine) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == blastFurnaceID)
		{
			return new GuiBlastFurnace(player, (TileBlastFurnace) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == alloySmelterID)
		{
			return new GuiAlloySmelter(player, (TileAlloySmelter) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == industrialGrinderID)
		{
			return new GuiIndustrialGrinder(player, (TileIndustrialGrinder) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == implosionCompresserID)
		{
			return new GuiImplosionCompressor(player,
					(TileImplosionCompressor) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == matterfabID)
		{
			return new GuiMatterFabricator(player, (TileMatterFabricator) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == chunkloaderID)
		{
			return new GuiChunkLoader(player, (TileChunkLoader) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == assemblingmachineID)
		{
			return new GuiAssemblingMachine(player, (TileAssemblingMachine) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == dieselGeneratorID)
		{
			return new GuiDieselGenerator(player, (TileDieselGenerator) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == industrialElectrolyzerID)
		{
			return new GuiIndustrialElectrolyzer(player,
					(TileIndustrialElectrolyzer) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == aesuID)
		{
			return new GuiAESU(player, (TileAesu) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == alloyFurnaceID)
		{
			return new GuiAlloyFurnace(player, (TileAlloyFurnace) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == sawMillID)
		{
			return new GuiIndustrialSawmill(player, (TileIndustrialSawmill) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == chemicalReactorID)
		{
			return new GuiChemicalReactor(player, (TileChemicalReactor) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == manuelID)
		{
			return new GuiManual();
		} else if (ID == destructoPackID)
		{
			return new GuiDestructoPack(new ContainerDestructoPack(player));
		} else if (ID == lesuID)
		{
			return new GuiLESU(player, (TileLesu) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == idsuID)
		{
			return new GuiIDSU(player, (TileIDSU) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == chargeBench)
		{
			return new GuiChargeBench(player, (TileChargeBench) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == fusionID)
		{
			return new GuiFusionReactor(player,
					(TileEntityFusionController) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == vacuumFreezerID)
		{
			return new GuiVacuumFreezer(player, (TileVacuumFreezer) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == grinderID)
		{
			return new GuiGrinder(player, (TileGrinder) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == generatorID)
		{
			return new GuiGenerator(player, (TileGenerator) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == extractorID)
		{
			return new GuiExtractor(player, (TileExtractor) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == compressorID)
		{
			return new GuiCompressor(player, (TileCompressor) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == electricFurnaceID)
		{
			return new GuiElectricFurnace(player, (TileElectricFurnace) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == ironFurnace)
		{
			return new GuiIronFurnace(player, (TileIronFurnace) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == recyclerID)
		{
			return new GuiRecycler(player, (TileRecycler) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == scrapboxinatorID)
		{
			return new GuiScrapboxinator(player, (TileScrapboxinator) world.func_175625_s(new BlockPos(x, y, z)));
		} else if (ID == batboxID)
		{
			return new GuiBatbox(player, (TileBatBox) world.func_175625_s(new BlockPos(x, y, z)));
		}
		else if (ID == mfsuID)
		{
			return new GuiMFSU(player, (TileMFSU) world.func_175625_s(new BlockPos(x, y, z)));
		}
		else if (ID == mfeID)
		{
			return new GuiMFE(player, (TileMFE) world.func_175625_s(new BlockPos(x, y, z)));
		}
		return null;
	}
}
