package techreborn.manual;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.inventory.Container;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL12;
import techreborn.init.ModBlocks;
import techreborn.init.ModItems;
import techreborn.items.ItemParts;
import techreborn.items.ItemPlates;
import techreborn.manual.loader.ManualLoader;
import techreborn.manual.pages.*;

import java.io.IOException;

@SideOnly(Side.CLIENT)
public class GuiManual extends GuiScreen
{
	protected final PageCollection root;
	public Container inventorySlots;
	protected int pageIndex = 0;
	protected int xSize = 0;
	protected int ySize = 0;
	protected int guiLeft;
	protected int guiTop;

	public GuiManual()
	{
		this.xSize = 200;
		this.ySize = 180;
		root = createRoot();
		//root = ManualLoader.getPages();
	}

	protected PageCollection createRoot()
	{
		pageIndex = 0;
		final PageCollection pageCollection = new PageCollection();
		pageCollection.addPage(new ContentsPage(Reference.pageNames.CONTENTS_PAGE, pageCollection));

		// GETTING STARTED
		pageCollection.addPage(new GettingStartedPage(Reference.pageNames.GETTINGSTARTED_PAGE, pageCollection));

		pageCollection.addPage(new DescriptionPage(Reference.pageNames.GETTINGRUBBER_PAGE, pageCollection, true,
				Reference.pageNames.GETTINGRUBBER2_PAGE));
		pageCollection.addPage(new CraftingInfoPage(Reference.pageNames.GETTINGRUBBER2_PAGE, pageCollection,
				ItemParts.getPartByName("rubber"), "", Reference.pageNames.GETTINGRUBBER_PAGE));
		pageCollection.addPage(new CraftingInfoPage(Reference.pageNames.CRAFTINGPLATES_PAGE, pageCollection,
				ItemPlates.getPlateByName("iron"), "", Reference.pageNames.GETTINGSTARTED_PAGE));

		// POWER GENERATION
		pageCollection.addPage(new GeneratingPowerPage(Reference.pageNames.GENERATINGPOWER_PAGE, pageCollection));

		pageCollection.addPage(new CraftingInfoPage(ModBlocks.Generator.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.Generator), "", Reference.pageNames.GENERATINGPOWER_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.thermalGenerator.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.thermalGenerator), "", Reference.pageNames.GENERATINGPOWER_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.solarPanel.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.solarPanel), "", Reference.pageNames.GENERATINGPOWER_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.heatGenerator.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.heatGenerator), "", Reference.pageNames.GENERATINGPOWER_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.LightningRod.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.LightningRod), "", Reference.pageNames.GENERATINGPOWER_PAGE));

		// BASIC MACHINES
		pageCollection.addPage(new BasicMachinesPage(Reference.pageNames.BASICMACHINES_PAGE, pageCollection));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.Grinder.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.Grinder), "", Reference.pageNames.BASICMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.ElectricFurnace.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.ElectricFurnace), "", Reference.pageNames.BASICMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.AlloySmelter.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.AlloySmelter), "", Reference.pageNames.BASICMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.Extractor.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.Extractor), "", Reference.pageNames.BASICMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.Compressor.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.Compressor), "", Reference.pageNames.BASICMACHINES_PAGE));

		// ADVANCED MACHINES
		pageCollection.addPage(new AdvancedMachines(Reference.pageNames.ADVANCEDMACHINES_PAGE, pageCollection));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.BlastFurnace.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.BlastFurnace), "", Reference.pageNames.ADVANCEDMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.industrialSawmill.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.industrialSawmill), "", Reference.pageNames.ADVANCEDMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.IndustrialElectrolyzer.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.IndustrialElectrolyzer), "", Reference.pageNames.ADVANCEDMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.IndustrialGrinder.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.IndustrialGrinder), "", Reference.pageNames.ADVANCEDMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.ImplosionCompressor.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.ImplosionCompressor), "", Reference.pageNames.ADVANCEDMACHINES_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModBlocks.centrifuge.func_149732_F(), pageCollection,
				new ItemStack(ModBlocks.centrifuge), "", Reference.pageNames.ADVANCEDMACHINES_PAGE));

		// TOOLS
		pageCollection.addPage(new ToolsPage(Reference.pageNames.TOOLS_PAGE, pageCollection));
		pageCollection.addPage(new CraftingInfoPage(ModItems.ironDrill.func_77658_a() + ".name", pageCollection,
				new ItemStack(ModItems.ironDrill), "", Reference.pageNames.TOOLS_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModItems.diamondDrill.func_77658_a() + ".name",
				pageCollection, new ItemStack(ModItems.diamondDrill), "", Reference.pageNames.TOOLS_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModItems.advancedDrill.func_77658_a() + ".name",
				pageCollection, new ItemStack(ModItems.advancedDrill), "", Reference.pageNames.TOOLS_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModItems.ironChainsaw.func_77658_a() + ".name",
				pageCollection, new ItemStack(ModItems.ironChainsaw), "", Reference.pageNames.TOOLS_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModItems.diamondChainsaw.func_77658_a() + ".name",
				pageCollection, new ItemStack(ModItems.diamondChainsaw), "", Reference.pageNames.TOOLS_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModItems.advancedChainsaw.func_77658_a() + ".name",
				pageCollection, new ItemStack(ModItems.advancedChainsaw), "", Reference.pageNames.TOOLS_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModItems.omniTool.func_77658_a() + ".name", pageCollection,
				new ItemStack(ModItems.omniTool), "", Reference.pageNames.TOOLS_PAGE));
		pageCollection.addPage(new CraftingInfoPage(ModItems.treeTap.func_77658_a() + ".name", pageCollection,
				new ItemStack(ModItems.treeTap), "", Reference.pageNames.TOOLS_PAGE));

		return pageCollection;
	}

	private int getNextPageIndex()
	{
		int i = pageIndex;
		pageIndex++;
		return i;
	}

	@Override
	public void func_73863_a(int mouseX, int mouseY, float par3)
	{
		drawGuiBackgroundLayer(par3, mouseX, mouseY);
		super.func_73863_a(mouseX, mouseY, par3);

		prepareRenderState();
		GL11.glPushMatrix();

		root.drawScreen(this.field_146297_k, this.guiLeft, this.guiTop, mouseX - this.guiLeft, mouseY - this.guiTop);

		GL11.glPopMatrix();
		restoreRenderState();
	}

	protected void prepareRenderState()
	{
		GL11.glDisable(GL12.GL_RESCALE_NORMAL);
		RenderHelper.func_74518_a();
		GL11.glDisable(GL11.GL_LIGHTING);
		GL11.glDisable(GL11.GL_DEPTH_TEST);
	}

	protected void restoreRenderState()
	{
		GL11.glEnable(GL12.GL_RESCALE_NORMAL);
		GL11.glEnable(GL11.GL_LIGHTING);
		GL11.glEnable(GL11.GL_DEPTH_TEST);
		RenderHelper.func_74519_b();
	}

	protected void drawGuiBackgroundLayer(float p_146976_1_, int mouseX, int mouseY)
	{
		GL11.glPushMatrix();
		GL11.glTranslated(this.guiLeft, this.guiTop, 0);
		root.renderBackgroundLayer(this.field_146297_k, 0, 0, mouseX - this.guiLeft, mouseY - this.guiTop);
		GL11.glPopMatrix();
	}

	@Override
	public void func_146280_a(Minecraft minecraft, int x, int y)
	{
		super.func_146280_a(minecraft, x, y);
		root.setWorldAndResolution(minecraft, x, y);
	}

	@Override
	public void func_146284_a(GuiButton button)
	{
		root.actionPerformed(button);
	}

	@Override
	public void func_73864_a(int par1, int par2, int par3) throws IOException
	{
		root.mouseClicked(par1, par2, par3);
	}

	@Override
	public void func_146269_k() throws IOException
	{
		super.func_146269_k();
	}

	@Override
	public void func_73866_w_()
	{
		super.func_73866_w_();
		this.guiLeft = (this.field_146294_l - this.xSize) / 2;
		this.guiTop = (this.field_146295_m - this.ySize) / 2;
	}

	@Override
	public boolean func_73868_f()
	{
		return false;
	}
}
