package techreborn.tiles.fusionReactor;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.TextComponentString;
import reborncore.api.power.EnumPowerTier;
import reborncore.api.power.tile.IEnergyProducerTile;
import reborncore.api.tile.IInventoryProvider;
import reborncore.common.explosion.RebornExplosion;
import reborncore.common.tile.TilePowerAcceptor;
import reborncore.common.util.Inventory;
import reborncore.common.util.ItemUtils;
import techreborn.api.reactor.FusionReactorRecipe;
import techreborn.api.reactor.FusionReactorRecipeHelper;
import techreborn.init.ModBlocks;
import techreborn.power.EnergyUtils;

public class TileEntityFusionController extends TilePowerAcceptor implements IEnergyProducerTile, IInventoryProvider {

	public Inventory inventory = new Inventory(3, "TileEntityFusionController", 64, this);

	public boolean hasCoils;

	public int recipeTickTime = 0;

	public FusionReactorRecipe recipe = null;
	public boolean repeatSameRecipe = false;

	@Override
	public double getMaxPower() {
		return 100000000;
	}

	@Override
	public boolean canAcceptEnergy(EnumFacing direction) {
		return recipeTickTime == 0;
	}

	@Override
	public boolean canProvideEnergy(EnumFacing direction) {
		return recipeTickTime != 0;
	}

	@Override
	public double getMaxOutput() {
		return 1000000;
	}

	@Override
	public EnumPowerTier getTier() {
		return EnumPowerTier.INSANE;
	}

	@Override
	public void func_145839_a(NBTTagCompound tagCompound) {
		super.func_145839_a(tagCompound);
		if(tagCompound.func_74764_b("recipeTickTime") && checkCoils()) {
			ItemStack topInput = ItemStack.func_77949_a(tagCompound.func_74775_l("TopInput"));
			ItemStack bottomInput = ItemStack.func_77949_a(tagCompound.func_74775_l("BottomInput"));
			FusionReactorRecipe newRecipe = findNewNBTRecipe(topInput, bottomInput);
			if(newRecipe != null) {
				this.recipeTickTime = tagCompound.func_74762_e("recipeTickTime");
				this.repeatSameRecipe = tagCompound.func_74767_n("repeatSameRecipe");
				this.recipe = newRecipe;
			}
		}
	}

	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound tagCompound) {
		super.func_189515_b(tagCompound);
		if(recipe != null) {
			tagCompound.func_74782_a("TopInput", recipe.getTopInput().func_77955_b(new NBTTagCompound()));
			tagCompound.func_74782_a("BottomInput", recipe.getBottomInput().func_77955_b(new NBTTagCompound()));

			tagCompound.func_74768_a("recipeTickTime", recipeTickTime);
			tagCompound.func_74757_a("repeatSameRecipe", repeatSameRecipe);
		}
		return tagCompound;
	}


	public boolean checkCoils() {
		if ((isCoil(this.func_174877_v().func_177958_n() + 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p()))
				&& (isCoil(this.func_174877_v().func_177958_n() + 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p()))
				&& (isCoil(this.func_174877_v().func_177958_n() - 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 3))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 3))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 3))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 3))
				&& (isCoil(this.func_174877_v().func_177958_n(), this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 3))
				&& (isCoil(this.func_174877_v().func_177958_n(), this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 3))) {
			hasCoils = true;
			return true;
		}
		hasCoils = false;
		return false;
	}

	private boolean isCoil(int x, int y, int z) {
		return field_145850_b.func_180495_p(new BlockPos(x, y, z)).func_177230_c() == ModBlocks.FusionCoil;
	}

	@Override
	public void func_73660_a() {
		super.func_73660_a();

		if (!field_145850_b.field_72995_K) {
			if (field_145850_b.func_82737_E() % 20 == 0) {
				checkCoils();
			}

			if (hasCoils) {
				if (recipe == null || inventory.hasChanged) {
					FusionReactorRecipe newRecipe = findNewRecipe();
					if (newRecipe != null && newRecipe != recipe) {
						this.recipe = newRecipe;
						this.repeatSameRecipe = false;
						this.recipeTickTime = 0;
					}
				}

				if (recipe != null) {
					if (recipeTickTime == 0) {
						if (canUseEnergy(recipe.getStartEU())) {
							useEnergy(recipe.getStartEU());
							consumeInputStacks();
							++recipeTickTime;
						}
					} else if (++recipeTickTime >= recipe.getTickTime()) {
						addOutputStack(recipe.getOutput());
						FusionReactorRecipe newRecipe = findNewRecipe();
						if (newRecipe == recipe) {
							consumeInputStacks();
							this.repeatSameRecipe = true;
							recipeTickTime = 1;
						} else {
							this.recipe = null;
							this.repeatSameRecipe = false;
							recipeTickTime = 0;
						}
					} else {
						double euTick = recipe.getEuTick();
						if (euTick > 0) addEnergy(euTick);
						else if (useEnergy(-euTick) != euTick) {
							this.recipeTickTime = 0;
							this.repeatSameRecipe = false;
							this.recipe = null;
						}
					}
				}
			}

			if (!field_145850_b.field_72995_K && getEnergy() > 0 && recipeTickTime != 0) {
				double maxOutput = getEnergy() > getMaxOutput() ? getMaxOutput() : getEnergy();
				double disposed = 0;
				for(EnumFacing facing : EnumFacing.field_82609_l) {
					if(canProvideEnergy(facing)) {
						double emitted = emitEnergy(facing, maxOutput);
						maxOutput -= emitted;
						disposed += emitted;
					}
				}
				if (disposed != 0)
					useEnergy(disposed);
			}
		}

	}

	public FusionReactorRecipe findNewRecipe() {
		for(FusionReactorRecipe reactorRecipe : FusionReactorRecipeHelper.reactorRecipes) {
			if(canFitStack(reactorRecipe.getOutput(), 2) &&
					canEmptyStack(reactorRecipe.getTopInput(), 0) &&
					canEmptyStack(reactorRecipe.getBottomInput(), 1)) {
				return reactorRecipe;
			}
		}
		return null;
	}

	public FusionReactorRecipe findNewNBTRecipe(ItemStack top, ItemStack bottom) {
		for(FusionReactorRecipe reactorRecipe : FusionReactorRecipeHelper.reactorRecipes) {
			if(canFitStack(reactorRecipe.getOutput(), 2) &&
					ItemUtils.isItemEqual(top, reactorRecipe.getTopInput(), true, true) &&
					ItemUtils.isItemEqual(bottom, reactorRecipe.getBottomInput(), true, true)) {
					return reactorRecipe;
			}
		}
		return null;
	}


	public double emitEnergy(EnumFacing enumFacing, double amount) {
		BlockPos pos = func_174877_v().func_177972_a(enumFacing);
		EnergyUtils.PowerNetReceiver receiver = EnergyUtils.getReceiver(
				field_145850_b, enumFacing.func_176734_d(), pos);
		if (receiver != null) {
			return receiver.receiveEnergy(amount, false);
		}
		return 0;
	}

	public void consumeInputStacks() {
		if(--func_70301_a(0).field_77994_a == 0) func_70299_a(0, null);
		if(--func_70301_a(1).field_77994_a == 0) func_70299_a(1, null);
	}

	public void addOutputStack(ItemStack output) {
		ItemStack stack = func_70301_a(2);
		if(stack == null) func_70299_a(2, output);
		else stack.field_77994_a++;
	}

	public boolean canEmptyStack(ItemStack stack, int slot) {
		return inventory.func_70301_a(slot) != null &&
				ItemUtils.isItemEqual(inventory.func_70301_a(slot), stack, true, true, true);
	}

	public boolean canFitStack(ItemStack stack, int slot) {
		if (stack == null || inventory.func_70301_a(slot) == null) {
			return true;
		}
		if (ItemUtils.isItemEqual(inventory.func_70301_a(slot), stack, true, true, true)) {
			if (stack.field_77994_a + inventory.func_70301_a(slot).field_77994_a <= stack.func_77976_d()) {
				return true;
			}
		}
		return false;
	}

	@Override
	public String func_70005_c_() {
		return "Fusion Reactor";
	}

	@Override
	public boolean func_145818_k_() {
		return false;
	}

	@Override
	public ITextComponent func_145748_c_() {
		return new TextComponentString(func_70005_c_());
	}

	@Override
	public Inventory getInventory() {
		return inventory;
	}

}
