package techreborn.tiles.generator;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.fluids.*;
import reborncore.api.power.EnumPowerTier;
import reborncore.api.tile.IInventoryProvider;
import reborncore.common.IWrenchable;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.tile.TilePowerProducer;
import reborncore.common.util.FluidUtils;
import reborncore.common.util.Inventory;
import reborncore.common.util.Tank;
import techreborn.config.ConfigTechReborn;
import techreborn.init.ModBlocks;
import techreborn.power.EnergyUtils;

public class TileThermalGenerator extends TilePowerProducer implements IWrenchable, IFluidHandler,IInventoryProvider {

	public static final int euTick = ConfigTechReborn.ThermalGeneratorOutput;
	public Tank tank = new Tank("TileThermalGenerator", FluidContainerRegistry.BUCKET_VOLUME * 10, this);
	public Inventory inventory = new Inventory(3, "TileThermalGenerator", 64, this);

	@Override
	public double emitEnergy(EnumFacing enumFacing, double amount) {
		BlockPos pos = func_174877_v().func_177972_a(enumFacing);
		EnergyUtils.PowerNetReceiver receiver = EnergyUtils.getReceiver(
				field_145850_b, enumFacing.func_176734_d(), pos);
		if(receiver != null) {
			addEnergy(amount - receiver.receiveEnergy(amount, false));
		} else addEnergy(amount);
		return 0; //Temporary hack die to my bug RebornCore
	}

	@Override
	public boolean wrenchCanSetFacing(EntityPlayer entityPlayer, EnumFacing side) {
		return false;
	}

	@Override
	public EnumFacing getFacing() {
		return getFacingEnum();
	}

	@Override
	public boolean wrenchCanRemove(EntityPlayer entityPlayer) {
		return entityPlayer.func_70093_af();
	}

	@Override
	public float getWrenchDropRate() {
		return 1.0F;
	}

	@Override
	public ItemStack getWrenchDrop(EntityPlayer entityPlayer) {
		return new ItemStack(ModBlocks.thermalGenerator, 1);
	}

	@Override
	public int fill(EnumFacing from, FluidStack resource, boolean doFill) {
		int fill = tank.fill(resource, doFill);
		tank.compareAndUpdate();
		return fill;
	}

	@Override
	public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain) {
		FluidStack drain = tank.drain(resource.amount, doDrain);
		tank.compareAndUpdate();
		return drain;
	}

	@Override
	public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain) {
		FluidStack drain = tank.drain(maxDrain, doDrain);
		tank.compareAndUpdate();
		return drain;
	}

	@Override
	public boolean canFill(EnumFacing from, Fluid fluid) {
		if (fluid != null) {
			if (fluid == FluidRegistry.LAVA) {
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean canDrain(EnumFacing from, Fluid fluid) {
		return tank.getFluid() == null || tank.getFluid().getFluid() == fluid;
	}

	@Override
	public FluidTankInfo[] getTankInfo(EnumFacing from) {
		return new FluidTankInfo[]{tank.getInfo()};
	}

	@Override
	public void func_145839_a(NBTTagCompound tagCompound) {
		super.func_145839_a(tagCompound);
		tank.readFromNBT(tagCompound);
	}

	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound tagCompound) {
		super.func_189515_b(tagCompound);
		tank.writeToNBT(tagCompound);
		return tagCompound;
	}

	@Override
	public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity packet) {
		field_145850_b.func_147458_c(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n(),
				func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
		func_145839_a(packet.func_148857_g());
	}

	@Override
	// TODO optimise this code
	public void func_73660_a() {
		super.func_73660_a();
		if (!field_145850_b.field_72995_K) {
			FluidUtils.drainContainers(this, inventory, 0, 1);
			for (EnumFacing direction : EnumFacing.values()) {
				if (field_145850_b.func_180495_p(new BlockPos(func_174877_v().func_177958_n() + direction.func_82601_c(),
						func_174877_v().func_177956_o() + direction.func_96559_d(), func_174877_v().func_177952_p() + direction.func_82599_e()))
						.func_177230_c() == Blocks.field_150353_l) {
					addEnergy(1);
				}
			}

			if (field_145850_b.func_82737_E() % 40 == 0) {
				BlockMachineBase bmb = (BlockMachineBase) field_145850_b.func_180495_p(field_174879_c).func_177230_c();
				boolean didFindLava = false;
				for (EnumFacing direction : EnumFacing.values()) {
					if (field_145850_b.func_180495_p(new BlockPos(func_174877_v().func_177958_n() + direction.func_82601_c(),
							func_174877_v().func_177956_o() + direction.func_96559_d(),
							func_174877_v().func_177952_p() + direction.func_82599_e())).func_177230_c() == Blocks.field_150353_l) {
						didFindLava = true;
					}
				}
				bmb.setActive(didFindLava, field_145850_b, field_174879_c);
			}
		}
		if (tank.getFluidAmount() > 0 && getMaxPower() - getEnergy() >= euTick) {
			tank.drain(1, true);
			addEnergy(euTick);
		}
		if (tank.getFluidType() != null && func_70301_a(2) == null) {
			// inventory.setInventorySlotContents(2, new ItemStack(tank
			// .getFluidType().getBlock()));
		} else if (tank.getFluidType() == null && func_70301_a(2) != null) {
			func_70299_a(2, null);
		}
	}

	@Override
	public double getMaxPower() {
		return 16000;
	}

	@Override
	public EnumPowerTier getTier() {
		return EnumPowerTier.LOW;
	}

	@Override
	public Inventory getInventory() {
		return inventory;
	}
}
