package techreborn.tiles.idsu;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import org.apache.commons.lang3.StringUtils;
import reborncore.api.power.EnumPowerTier;
import reborncore.common.IWrenchable;
import reborncore.common.tile.TilePowerAcceptorProducer;
import techreborn.config.ConfigTechReborn;
import techreborn.init.ModBlocks;
import techreborn.power.EnergyUtils;

public class TileIDSU extends TilePowerAcceptorProducer implements IWrenchable {

	public String ownerUdid;
	public int tier;
	public int output;
	public double maxStorage;
	private double euLastTick = 0;
	private double euChange;
	private int ticks;

	public TileIDSU(int tier1, int output1, int maxStorage1) {
		this.tier = tier1;
		this.output = output1;
		this.maxStorage = maxStorage1;
	}

	public TileIDSU() {
		this(5, 2048, 100000000);
	}

	@Override
	public double getEnergy() {
		if (ownerUdid == null && StringUtils.isBlank(ownerUdid) || StringUtils.isEmpty(ownerUdid)) {
			return 0.0;
		}
		return IDSUManager.INSTANCE.getSaveDataForWorld(field_145850_b, ownerUdid).storedPower;
	}

	@Override
	public void setEnergy(double energy) {
		if (ownerUdid == null && StringUtils.isBlank(ownerUdid) || StringUtils.isEmpty(ownerUdid)) {
			return;
		}
		IDSUManager.INSTANCE.getSaveDataForWorld(field_145850_b, ownerUdid).storedPower = energy;
	}

	@Override
	public double getMaxPower() {
		return 1000000000;
	}

	@Override
	public EnumPowerTier getTier() {
		return EnumPowerTier.INSANE;
	}

	public float getChargeLevel() {
		float ret = (float) this.getEnergy() / (float) this.maxStorage;
		if (ret > 1.0F) {
			ret = 1.0F;
		}

		return ret;
	}

	public void func_145839_a(NBTTagCompound nbttagcompound) {
		super.func_145839_a(nbttagcompound);
		this.ownerUdid = nbttagcompound.func_74779_i("ownerUdid");
	}

	public NBTTagCompound func_189515_b(NBTTagCompound nbttagcompound) {
		super.func_189515_b(nbttagcompound);
		if (ownerUdid == null || StringUtils.isEmpty(ownerUdid)) {
			return nbttagcompound;
		}
		nbttagcompound.func_74778_a("ownerUdid", this.ownerUdid);
		return nbttagcompound;
	}

	public void func_73660_a() {
		super.func_73660_a();

		if (ticks == ConfigTechReborn.AverageEuOutTickTime) {
			euChange = -1;
			ticks = 0;

		} else {
			ticks++;
			euChange += getEnergy() - euLastTick;
			if (euLastTick == getEnergy()) {
				euChange = 0;
			}
		}

		euLastTick = getEnergy();

		if (!field_145850_b.field_72995_K && getEnergy() > 0) {
			double maxOutput = getEnergy() > getMaxOutput() ? getMaxOutput() : getEnergy();
			double disposed = emitEnergy(getFacingEnum(), maxOutput);
			if (maxOutput != 0 && disposed != 0) useEnergy(disposed);
		}
	}

	public double emitEnergy(EnumFacing enumFacing, double amount) {
		BlockPos pos = func_174877_v().func_177972_a(enumFacing);
		EnergyUtils.PowerNetReceiver receiver = EnergyUtils.getReceiver(
				field_145850_b, enumFacing.func_176734_d(), pos);
		if(receiver != null) {
			return receiver.receiveEnergy(amount, false);
		}
		return 0;
	}

	@Override
	public boolean canAcceptEnergy(EnumFacing direction) {
		return getFacingEnum() != direction;
	}

	@Override
	public boolean canProvideEnergy(EnumFacing direction) {
		return getFacingEnum() == direction;
	}


	public boolean wrenchCanSetFacing(EntityPlayer entityPlayer, int side) {
		return false;
	}

	@Override
	public boolean wrenchCanSetFacing(EntityPlayer p0, EnumFacing p1) {
		return true;
	}

	@Override
	public EnumFacing getFacing() {
		return getFacingEnum();
	}

	@Override
	public boolean wrenchCanRemove(EntityPlayer p0) {
		return true;
	}

	@Override
	public float getWrenchDropRate() {
		return 1.0F;
	}

	public ItemStack getWrenchDrop(EntityPlayer entityPlayer) {
		NBTTagCompound tileEntity = new NBTTagCompound();
		ItemStack dropStack = new ItemStack(ModBlocks.Idsu, 1);
		func_189515_b(tileEntity);
		dropStack.func_77982_d(new NBTTagCompound());
		dropStack.func_77978_p().func_74782_a("tileEntity", tileEntity);
		return dropStack;
	}

	public double getEuChange() {
		if (euChange == -1) {
			return -1;
		}
		return (euChange / ticks);
	}

	public void handleGuiInputFromClient(int id) {
		if (id == 0) {
			output += 256;
		}
		if (id == 1) {
			output += 64;
		}
		if (id == 2) {
			output -= 64;
		}
		if (id == 3) {
			output -= 256;
		}
		if (output > 4096) {
			output = 4096;
		}
		if (output <= -1) {
			output = 0;
		}
	}

}
