package techreborn.tiles.multiblock;

import net.minecraft.block.state.IBlockState;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import techreborn.blocks.BlockMachineCasing;
import techreborn.init.ModBlocks;

public class MultiblockChecker {

    public static final BlockPos ZERO_OFFSET = BlockPos.field_177992_a;

    public static final int CASING_NORMAL = 0;
    public static final int CASING_REINFORCED = 1;
    public static final int CASING_ADVANCED = 2;

    private final World world;
    private final BlockPos downCenter;

    public MultiblockChecker(World world, BlockPos downCenter) {
        this.world = world;
        this.downCenter = downCenter;
    }

    public boolean checkCasing(int offX, int offY, int offZ, int type) {
        IBlockState block = getBlock(offX, offY, offZ);
        if(block.func_177230_c() == ModBlocks.MachineCasing) {
          if(block.func_177229_b(BlockMachineCasing.METADATA) == type)
              return true;
        }
        return false;
    }

    public boolean checkAir(int offX, int offY, int offZ) {
        BlockPos pos = downCenter.func_177982_a(offX, offY, offZ);
        return world.func_175623_d(pos);
    }

    public IBlockState getBlock(int offX, int offY, int offZ) {
        BlockPos pos = downCenter.func_177982_a(offX, offY, offZ);
        return world.func_180495_p(pos);
    }

    public boolean checkRectY(int sizeX, int sizeZ, int casingType, BlockPos offset) {
        for(int x = -sizeX; x <= sizeX; x++) {
            for(int z = -sizeZ; z <= sizeZ; z++) {
                if(!checkCasing(x + offset.func_177958_n(), offset.func_177956_o(), z + offset.func_177952_p(), casingType))
                    return false;
            }
        }
        return true;
    }

    public boolean checkRectZ(int sizeX, int sizeY, int casingType, BlockPos offset) {
        for(int x = -sizeX; x <= sizeX; x++) {
            for(int y = -sizeY; y <= sizeY; y++) {
                if(!checkCasing(x + offset.func_177958_n(), y + offset.func_177956_o(), offset.func_177952_p(), casingType))
                    return false;
            }
        }
        return true;
    }

    public boolean checkRectX(int sizeZ, int sizeY, int casingType, BlockPos offset) {
        for(int z = -sizeZ; z <= sizeZ; z++) {
            for(int y = -sizeY; y <= sizeY; y++) {
                if(!checkCasing(offset.func_177958_n(), y + offset.func_177956_o(), z + offset.func_177952_p(), casingType))
                    return false;
            }
        }
        return true;
    }

    public boolean checkRingY(int sizeX, int sizeZ, int casingType, BlockPos offset) {
        for(int x = -sizeX; x <= sizeX; x++) {
            for(int z = -sizeZ; z <= sizeZ; z++) {
                if((x == sizeX || x == -sizeX) || (z == sizeZ || z == -sizeZ)) {
                    if (!checkCasing(x + offset.func_177958_n(), offset.func_177956_o(), z + offset.func_177952_p(), casingType))
                        return false;
                }
            }
        }
        return true;
    }

    public boolean checkRingYHollow(int sizeX, int sizeZ, int casingType, BlockPos offset) {
        for(int x = -sizeX; x <= sizeX; x++) {
            for(int z = -sizeZ; z <= sizeZ; z++) {
                if((x == sizeX || x == -sizeX) || (z == sizeZ || z == -sizeZ)) {
                    if (!checkCasing(x + offset.func_177958_n(), offset.func_177956_o(), z + offset.func_177952_p(), casingType))
                        return false;
                } else if (!checkAir(x + offset.func_177958_n(), offset.func_177956_o(), z + offset.func_177952_p()))
                    return false;
            }
        }
        return true;
    }

}
